@@ -27,19 +27,26 @@
 #endif
 
 #define __LOCATION__ cp__l(__SHORT_FILE__,__LINE__)
-#define CPWARN(msg) CALL cp__w(__SHORT_FILE__,__LINE__,msg)
-#define CPABORT(msg) CALL cp__b(__SHORT_FILE__,__LINE__,msg)
-! In contrast to CPWARN, the warning counter is not increased
-#define CPHINT(msg) CALL cp__h(__SHORT_FILE__,__LINE__,msg)
+#define CPABORT(MSG) CALL cp__b(__SHORT_FILE__,__LINE__,MSG)
 
-# define CPASSERT(cond) IF(.NOT.(cond))CALL cp__a(__SHORT_FILE__,__LINE__)
+! Issue a warning; warnings are summarized globally.
+! For conditional warnings see CPWARN_IF.
+#define CPWARN(MSG) CALL cp__w(__SHORT_FILE__,__LINE__,MSG)
+
+! Like CPWARN but only if CONDition is true.
+#define CPWARN_IF(COND, MSG) IF(COND)CPWARN(MSG)
+
+! In contrast to CPWARN, the warning counter is not increased
+#define CPHINT(MSG) CALL cp__h(__SHORT_FILE__,__LINE__,MSG)
+
+# define CPASSERT(COND) IF(.NOT.(COND))CALL cp__a(__SHORT_FILE__,__LINE__)
 
 ! The MARK_USED macro can be used to mark an argument/variable as used. It is intended to make
 ! it possible to switch on -Werror=unused-dummy-argument, but deal elegantly with, e.g.,
 ! library wrapper routines that take arguments only used if the library is linked in.
 ! This code should be valid for any Fortran variable, is always standard conforming,
 ! and will be optimized away completely by the compiler
-#define MARK_USED(foo) IF(.FALSE.)THEN; DO ; IF(SIZE(SHAPE(foo))==-1) EXIT ;  END DO ; ENDIF
+#define MARK_USED(FOO) IF(.FALSE.)THEN;DO;IF(SIZE(SHAPE(FOO))==-1) EXIT;ENDDO;ENDIF
 
 ! Calculate version number from 2 or 3 components. Can be used for comparison, e.g.,
 ! CPVERSION3(4, 9, 0) <= CPVERSION3(__GNUC__, __GNUC_MINOR__, __GNUC_PATCHLEVEL__)
@@ -48,10 +55,23 @@
 #define CPVERSION3(MAJOR, MINOR, UPDATE) (CPVERSION2(MAJOR, MINOR) + (UPDATE))
 #define CPVERSION CPVERSION2
 
+! On top of CPVERSION macro, test if MAJOR_TEST and MINOR_TEST are defined.
+! Usage:  #if CPVERSION_CHECK(9, 5, >, __GNUC__, __GNUC_MINOR__)
+! Perform actual comparison according to COMP argument.
+! Note: defined(MAJOR_TEST) and defined(MINOR_TEST) is avoided in macro
+!       definition due to issues handling it in certain compilers.
+#define CPVERSION_CHECK(MAJOR_BASE, MINOR_BASE, COMP, MAJOR_TEST, MINOR_TEST) ((MAJOR_TEST) && \
+  (CPVERSION2(MAJOR_BASE, MINOR_BASE) COMP CPVERSION2(MAJOR_TEST, MINOR_TEST)))
+
+! Avoid to default initialize type-components (default c'tor)
+#if CPVERSION_CHECK(9, 5, >, __GNUC__, __GNUC_MINOR__) || defined(__INTEL_COMPILER) || defined(__INTEL_LLVM_COMPILER)
+#define FTN_NO_DEFAULT_INIT
+#endif
+
 ! gfortran before 8.3 complains about internal symbols not being specified in
 ! any data clause when using DEFAULT(NONE) and OOP procedures are called from
 ! within the parallel region.
-#if __GNUC__ < 8 || (__GNUC__ == 8 && (__GNUC_MINOR__ < 3))
+#if CPVERSION_CHECK(8, 3, >, __GNUC__, __GNUC_MINOR__)
 #define OMP_DEFAULT_NONE_WITH_OOP SHARED
 #else
 #define OMP_DEFAULT_NONE_WITH_OOP NONE
