@@ -8,15 +8,22 @@
 #ifndef DBM_HYPERPARAMS_H
 #define DBM_HYPERPARAMS_H
 
-// TODO: Tune parameters, check if dynamic OpenMP scheduling is really faster?
+#define DBM_OMP_SCHEDULE schedule(dynamic, 1)
 
-static const float HASHTABLE_FACTOR = 3.0;
-static const float ALLOCATION_FACTOR = 1.5;
-static const float SHARDS_PER_THREAD = 1.0;
-static const int MAX_BATCH_SIZE = 10000;
-static const int BATCH_NUM_BUCKETS = 1000;
-static const int INITIAL_NBLOCKS_ALLOCATED = 100;
-static const int INITIAL_DATA_ALLOCATED = 1024;
+#define DBM_MEMPOOL_DEVICE 1
+#define DBM_MEMPOOL_HOST 1
+
+#define DBM_ALLOC_OFFLOAD 1
+#define DBM_ALLOC_OPENMP 1
+#define DBM_ALLOC_MPI 0
+
+#define DBM_OVERCOMMIT_DEVICE 1.3
+#define DBM_OVERCOMMIT_HOST 1.4
+#define DBM_HASHTABLE_FACTOR 3.0
+
+#define DBM_SHARDS_PER_THREAD 1.0
+#define DBM_MAX_BATCH_SIZE 30000
+#define DBM_BATCH_NUM_BUCKETS 1000
 
 #endif
 
