@@ -8,7 +8,6 @@
 #include <assert.h>
 #include <inttypes.h>
 #include <omp.h>
-#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -131,8 +130,7 @@ void dbm_library_print_stats(const int fortran_comm,
 
   const dbm_mpi_comm_t comm = dbm_mpi_comm_f2c(fortran_comm);
   // Sum all counters across threads and mpi ranks.
-  int64_t counters[DBM_NUM_COUNTERS][2];
-  memset(counters, 0, DBM_NUM_COUNTERS * 2 * sizeof(int64_t));
+  int64_t counters[DBM_NUM_COUNTERS][2] = {{0}};
   double total = 0.0;
   for (int i = 0; i < DBM_NUM_COUNTERS; i++) {
     counters[i][1] = i; // needed as inverse index after qsort
@@ -168,6 +166,7 @@ void dbm_library_print_stats(const int fortran_comm,
              output_unit);
 
   const char *labels[] = {"?", "??", "???", ">999"};
+  char buffer[100];
   for (int i = 0; i < DBM_NUM_COUNTERS; i++) {
     if (counters[i][0] == 0) {
       continue; // skip empty counters
@@ -177,7 +176,6 @@ void dbm_library_print_stats(const int fortran_comm,
     const int m = (idx % 64) / 16;
     const int n = (idx % 16) / 4;
     const int k = (idx % 4) / 1;
-    char buffer[100];
     snprintf(buffer, sizeof(buffer),
              " %4s  x %4s  x %4s %46" PRId64 " %10.2f%%\n", labels[m],
              labels[n], labels[k], counters[i][0], percent);
@@ -187,6 +185,43 @@ void dbm_library_print_stats(const int fortran_comm,
   print_func(" ----------------------------------------------------------------"
              "---------------\n",
              output_unit);
+
+  dbm_memstats_t memstats;
+  dbm_mempool_statistics(&memstats);
+  dbm_mpi_max_uint64(&memstats.device_mallocs, 1, comm);
+  dbm_mpi_max_uint64(&memstats.host_mallocs, 1, comm);
+
+  if (0 != memstats.device_mallocs || 0 != memstats.host_mallocs) {
+    print_func(" Memory consumption               "
+               " Number of allocations  Used [MiB]  Size [MiB]\n",
+               output_unit);
+  }
+  if (0 < memstats.device_mallocs) {
+    dbm_mpi_max_uint64(&memstats.device_size, 1, comm);
+    snprintf(buffer, sizeof(buffer),
+             " Device                            "
+             " %20" PRIuPTR "  %10" PRIuPTR "  %10" PRIuPTR "\n",
+             (uintptr_t)memstats.device_mallocs,
+             (uintptr_t)((memstats.device_used + (512U << 10)) >> 20),
+             (uintptr_t)((memstats.device_size + (512U << 10)) >> 20));
+    print_func(buffer, output_unit);
+  }
+  if (0 < memstats.host_mallocs) {
+    dbm_mpi_max_uint64(&memstats.host_size, 1, comm);
+    snprintf(buffer, sizeof(buffer),
+             " Host                              "
+             " %20" PRIuPTR "  %10" PRIuPTR "  %10" PRIuPTR "\n",
+             (uintptr_t)memstats.host_mallocs,
+             (uintptr_t)((memstats.host_used + (512U << 10)) >> 20),
+             (uintptr_t)((memstats.host_size + (512U << 10)) >> 20));
+    print_func(buffer, output_unit);
+  }
+  if (0 < memstats.device_mallocs || 0 < memstats.host_mallocs) {
+    print_func(
+        " ----------------------------------------------------------------"
+        "---------------\n",
+        output_unit);
+  }
 }
 
 // EOF
