@@ -34,7 +34,7 @@ void dbm_create(dbm_matrix_t **matrix_out, dbm_distribution_t *dist,
 
   size_t size = (strlen(name) + 1) * sizeof(char);
   matrix->name = malloc(size);
-  assert(matrix->name != NULL);
+  assert(matrix->name != NULL && name != NULL);
   memcpy(matrix->name, name, size);
 
   matrix->nrows = nrows;
@@ -42,18 +42,23 @@ void dbm_create(dbm_matrix_t **matrix_out, dbm_distribution_t *dist,
 
   size = nrows * sizeof(int);
   matrix->row_sizes = malloc(size);
-  assert(matrix->row_sizes != NULL);
-  memcpy(matrix->row_sizes, row_sizes, size);
+  assert(matrix->row_sizes != NULL || size == 0);
+  if (size != 0) {
+    memcpy(matrix->row_sizes, row_sizes, size);
+  }
 
   size = ncols * sizeof(int);
   matrix->col_sizes = malloc(size);
-  assert(matrix->col_sizes != NULL);
-  memcpy(matrix->col_sizes, col_sizes, size);
+  assert(matrix->col_sizes != NULL || size == 0);
+  if (size != 0) {
+    memcpy(matrix->col_sizes, col_sizes, size);
+  }
 
-  matrix->shards = malloc(dbm_get_num_shards(matrix) * sizeof(dbm_shard_t));
-  assert(matrix->shards != NULL);
+  const int num_shards = dbm_get_num_shards(matrix);
+  matrix->shards = malloc(num_shards * sizeof(dbm_shard_t));
+  assert(matrix->shards != NULL || num_shards == 0);
 #pragma omp parallel for
-  for (int ishard = 0; ishard < dbm_get_num_shards(matrix); ishard++) {
+  for (int ishard = 0; ishard < num_shards; ishard++) {
     dbm_shard_init(&matrix->shards[ishard]);
   }
 
@@ -97,7 +102,7 @@ void dbm_copy(dbm_matrix_t *matrix_a, const dbm_matrix_t *matrix_b) {
 
   assert(matrix_a->dist == matrix_b->dist);
 
-#pragma omp parallel for schedule(dynamic)
+#pragma omp parallel for DBM_OMP_SCHEDULE
   for (int ishard = 0; ishard < dbm_get_num_shards(matrix_a); ishard++) {
     dbm_shard_copy(&matrix_a->shards[ishard], &matrix_b->shards[ishard]);
   }
@@ -244,10 +249,11 @@ void dbm_put_block(dbm_matrix_t *matrix, const int row, const int col,
       dbm_shard_get_or_allocate_block(shard, row, col, block_size);
   double *blk_data = &shard->data[blk->offset];
   if (summation) {
+    assert(blk_data != NULL || 0 == block_size);
     for (int i = 0; i < block_size; i++) {
       blk_data[i] += block[i];
     }
-  } else {
+  } else if (block_size != 0) {
     memcpy(blk_data, block, block_size * sizeof(double));
   }
   omp_unset_lock(&shard->lock);
@@ -260,7 +266,7 @@ void dbm_put_block(dbm_matrix_t *matrix, const int row, const int col,
 void dbm_clear(dbm_matrix_t *matrix) {
   assert(omp_get_num_threads() == 1);
 
-#pragma omp parallel for schedule(dynamic)
+#pragma omp parallel for DBM_OMP_SCHEDULE
   for (int ishard = 0; ishard < dbm_get_num_shards(matrix); ishard++) {
     dbm_shard_t *shard = &matrix->shards[ishard];
     shard->nblocks = 0;
@@ -284,7 +290,7 @@ void dbm_filter(dbm_matrix_t *matrix, const double eps) {
   }
   const double eps2 = eps * eps;
 
-#pragma omp parallel for schedule(dynamic)
+#pragma omp parallel for DBM_OMP_SCHEDULE
   for (int ishard = 0; ishard < dbm_get_num_shards(matrix); ishard++) {
     dbm_shard_t *shard = &matrix->shards[ishard];
     const int old_nblocks = shard->nblocks;
@@ -300,7 +306,11 @@ void dbm_filter(dbm_matrix_t *matrix, const double eps) {
       const int block_size = row_size * col_size;
       double norm = 0.0;
       for (int i = 0; i < block_size; i++) {
-        norm += old_blk_data[i] * old_blk_data[i];
+        const double value = old_blk_data[i];
+        norm += value * value;
+        if (eps2 <= norm) {
+          break;
+        }
       }
       // For historic reasons zero-sized blocks are never filtered.
       if (block_size > 0 && norm < eps2) {
@@ -311,7 +321,7 @@ void dbm_filter(dbm_matrix_t *matrix, const double eps) {
           shard, old_blk.row, old_blk.col, block_size);
       assert(new_blk->offset <= old_blk.offset);
       if (new_blk->offset != old_blk.offset) {
-        // Using memmove instead of memcpy because it handles overlap correctly.
+        // Using memmove because it can handle overlapping buffers.
         double *new_blk_data = &shard->data[new_blk->offset];
         memmove(new_blk_data, old_blk_data, block_size * sizeof(double));
       }
@@ -346,7 +356,7 @@ void dbm_reserve_blocks(dbm_matrix_t *matrix, const int nblocks,
   }
 #pragma omp barrier
 
-#pragma omp for schedule(dynamic)
+#pragma omp for DBM_OMP_SCHEDULE
   for (int ishard = 0; ishard < dbm_get_num_shards(matrix); ishard++) {
     dbm_shard_t *shard = &matrix->shards[ishard];
     dbm_shard_allocate_promised_blocks(shard);
@@ -367,7 +377,7 @@ void dbm_scale(dbm_matrix_t *matrix, const double alpha) {
     return;
   }
 
-#pragma omp parallel for schedule(dynamic)
+#pragma omp parallel for DBM_OMP_SCHEDULE
   for (int ishard = 0; ishard < dbm_get_num_shards(matrix); ishard++) {
     dbm_shard_t *shard = &matrix->shards[ishard];
     for (int i = 0; i < shard->data_size; i++) {
@@ -383,10 +393,12 @@ void dbm_scale(dbm_matrix_t *matrix, const double alpha) {
 void dbm_zero(dbm_matrix_t *matrix) {
   assert(omp_get_num_threads() == 1);
 
-#pragma omp parallel for schedule(dynamic)
+#pragma omp parallel for DBM_OMP_SCHEDULE
   for (int ishard = 0; ishard < dbm_get_num_shards(matrix); ishard++) {
     dbm_shard_t *shard = &matrix->shards[ishard];
-    memset(shard->data, 0, shard->data_size * sizeof(double));
+    if (shard->data != NULL) {
+      memset(shard->data, 0, shard->data_size * sizeof(double));
+    }
   }
 }
 
@@ -398,7 +410,7 @@ void dbm_add(dbm_matrix_t *matrix_a, const dbm_matrix_t *matrix_b) {
   assert(omp_get_num_threads() == 1);
   assert(matrix_a->dist == matrix_b->dist);
 
-#pragma omp parallel for schedule(dynamic)
+#pragma omp parallel for DBM_OMP_SCHEDULE
   for (int ishard = 0; ishard < dbm_get_num_shards(matrix_b); ishard++) {
     dbm_shard_t *shard_a = &matrix_a->shards[ishard];
     const dbm_shard_t *shard_b = &matrix_b->shards[ishard];
@@ -501,16 +513,32 @@ void dbm_iterator_next_block(dbm_iterator_t *iter, int *row, int *col,
  ******************************************************************************/
 void dbm_iterator_stop(dbm_iterator_t *iter) { free(iter); }
 
+/*******************************************************************************
+ * \brief Private routine to accumulate using Kahan's summation.
+ * \author Hans Pabst
+ ******************************************************************************/
+static double kahan_sum(double value, double *accumulator,
+                        double *compensation) {
+  double r, c;
+  assert(NULL != accumulator && NULL != compensation);
+  c = value - *compensation;
+  r = *accumulator + c;
+  *compensation = (r - *accumulator) - c;
+  *accumulator = r;
+  return r;
+}
+
 /*******************************************************************************
  * \brief Computes a checksum of the given matrix.
  * \author Ole Schuett
  ******************************************************************************/
 double dbm_checksum(const dbm_matrix_t *matrix) {
-  double checksum = 0.0;
+  double checksum = 0.0, compensation = 0.0;
   for (int ishard = 0; ishard < dbm_get_num_shards(matrix); ishard++) {
     const dbm_shard_t *shard = &matrix->shards[ishard];
     for (int i = 0; i < shard->data_size; i++) {
-      checksum += shard->data[i] * shard->data[i];
+      const double value = shard->data[i];
+      kahan_sum(value * value, &checksum, &compensation);
     }
   }
   dbm_mpi_sum_double(&checksum, 1, matrix->dist->comm);
@@ -518,13 +546,13 @@ double dbm_checksum(const dbm_matrix_t *matrix) {
 }
 
 /*******************************************************************************
- * \brief Returns the absolute value of the larges element of the entire matrix.
+ * \brief Returns the largest absolute value of all matrix elements.
  * \author Ole Schuett
  ******************************************************************************/
 double dbm_maxabs(const dbm_matrix_t *matrix) {
   double maxabs = 0.0;
   for (int ishard = 0; ishard < dbm_get_num_shards(matrix); ishard++) {
-    dbm_shard_t *shard = &matrix->shards[ishard];
+    const dbm_shard_t *shard = &matrix->shards[ishard];
     for (int i = 0; i < shard->data_size; i++) {
       maxabs = fmax(maxabs, fabs(shard->data[i]));
     }
