@@ -11,7 +11,7 @@
 #include "../offload/offload_runtime.h"
 #if defined(__OFFLOAD) && !defined(__NO_OFFLOAD_DBM)
 
-#include "dbm_multiply_internal.h"
+#include "dbm_internal.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -22,10 +22,12 @@ extern "C" {
  *        All arguments are assumed to be device pointers.
  * \author Ole Schuett
  ******************************************************************************/
-void dbm_multiply_gpu_launch_kernel(
-    const offloadStream_t stream, const int mnk_range[3][2], const double alpha,
-    const int ntasks, const dbm_task_t *batch, const double *pack_a_data,
-    const double *pack_b_data, double *shard_c_data);
+void dbm_multiply_gpu_launch_kernel(offloadStream_t stream, double alpha,
+                                    int ntasks, const dbm_task_t *tasks_host,
+                                    const dbm_task_t *tasks,
+                                    const double *pack_a_data,
+                                    const double *pack_b_data,
+                                    double *shard_c_data);
 
 #ifdef __cplusplus
 }
