@@ -326,6 +326,11 @@ bool grid_replay(const char *filename, const int cycles, const bool collocate,
     abort();
   }
 
+  if (fclose(fp) != 0) {
+    fprintf(stderr, "Could not close task file: %s\n", filename);
+    abort();
+  }
+
   offload_buffer *grid_test = NULL;
   offload_create_buffer(npts_local_total, &grid_test);
   double hab_test[n2][n1];
