@@ -2,10 +2,10 @@
 
 # author: Ole Schuett
 
-from pathlib import Path
-from typing import Any
 import argparse
 import io
+from pathlib import Path
+from typing import Any
 
 
 # ======================================================================================
@@ -16,109 +16,113 @@ def main() -> None:
 
     for version in "sdbg", "ssmp", "pdbg", "psmp":
         with OutputFile(f"Dockerfile.test_{version}", args.check) as f:
-            if version == "ssmp":
-                # Use ssmp as guinea pig
-                f.write(toolchain_full(with_dbcsr=True))
-                f.write(regtest_cmake("toolchain", version))
-            else:
-                f.write(toolchain_full() + regtest(version))
+            mpi_mode = "mpich" if version.startswith("p") else "no"
+            f.write(install_deps_toolchain(mpi_mode=mpi_mode, with_dbcsr=""))
+            f.write(regtest_cmake("toolchain_all", version))
 
-        with OutputFile(f"Dockerfile.test_generic_{version}", args.check) as f:
-            f.write(toolchain_full(target_cpu="generic") + regtest(version))
+    with OutputFile(f"Dockerfile.test_generic_psmp", args.check) as f:
+        f.write(install_deps_toolchain(target_cpu="generic", with_dbcsr=""))
+        f.write(regtest_cmake("toolchain_generic", "psmp"))
 
     with OutputFile(f"Dockerfile.test_openmpi-psmp", args.check) as f:
-        f.write(toolchain_full(mpi_mode="openmpi"))
-        f.write(regtest("psmp"))
+        f.write(install_deps_toolchain(mpi_mode="openmpi", with_dbcsr=""))
+        f.write(regtest_cmake("toolchain_all", "psmp"))
 
     with OutputFile(f"Dockerfile.test_fedora-psmp", args.check) as f:
-        f.write(toolchain_full(base_image="fedora:38") + regtest("psmp"))
+        f.write(install_deps_toolchain(base_image="fedora:41", with_dbcsr=""))
+        f.write(regtest_cmake("toolchain_all", "psmp"))
 
-    with OutputFile(f"Dockerfile.test_intel-psmp", args.check) as f:
-        f.write(
-            toolchain_intel()
-            + regtest("psmp", intel=True, testopts="--mpiexec mpiexec")
-        )
+    for ver in "ssmp", "psmp":
+        with OutputFile(f"Dockerfile.test_intel-{ver}", args.check) as f:
+            base_image = "intel/hpckit:2024.2.1-0-devel-ubuntu22.04"
+            f.write(install_deps_toolchain_intel(base_image=base_image, with_ifx="no"))
+            f.write(regtest(ver, intel=True, testopts="--mpiexec mpiexec"))
+        with OutputFile(f"Dockerfile.test_intel-oneapi-hpckit-{ver}", args.check) as f:
+            base_image = "intel/oneapi-hpckit:2025.1.3-0-devel-ubuntu24.04"
+            f.write(install_deps_toolchain_intel(base_image=base_image, with_ifx="yes"))
+            f.write(regtest(ver, intel=True, testopts="--mpiexec mpiexec"))
 
     with OutputFile(f"Dockerfile.test_nvhpc", args.check) as f:
-        f.write(toolchain_nvhpc())
+        f.write(install_deps_toolchain_nvhpc())
 
     with OutputFile(f"Dockerfile.test_minimal", args.check) as f:
-        f.write(toolchain_ubuntu_nompi() + regtest_cmake("minimal", "ssmp"))
+        f.write(install_deps_ubuntu())
+        f.write(install_dbcsr("minimal", "ssmp"))
+        f.write(regtest_cmake("minimal", "ssmp"))
 
     with OutputFile(f"Dockerfile.test_spack", args.check) as f:
-        f.write(spack_env_toolchain() + regtest_cmake("spack", "psmp"))
+        f.write(install_deps_spack("psmp"))
+        f.write(regtest_cmake("spack_all", "psmp"))
 
-    for version in "ssmp", "psmp":
-        with OutputFile(f"Dockerfile.test_asan-{version}", args.check) as f:
-            f.write(toolchain_full() + regtest(version, "local_asan"))
+    with OutputFile(f"Dockerfile.test_asan-psmp", args.check) as f:
+        f.write(install_deps_toolchain(with_dbcsr=""))
+        f.write(regtest_cmake("toolchain_asan", "psmp"))
 
     for version in "sdbg", "pdbg":
         with OutputFile(f"Dockerfile.test_coverage-{version}", args.check) as f:
-            f.write(toolchain_full() + coverage(version))
+            f.write(install_deps_toolchain())
+            f.write(coverage(version))
 
     for gcc_version in 8, 9, 10, 11, 12, 13, 14:
         with OutputFile(f"Dockerfile.test_gcc{gcc_version}", args.check) as f:
             if gcc_version > 8:
-                f.write(toolchain_ubuntu_nompi(gcc_version=gcc_version))
+                f.write(install_deps_ubuntu(gcc_version=gcc_version))
+                f.write(install_dbcsr("ubuntu", "ssmp"))
                 f.write(regtest_cmake("ubuntu", "ssmp"))
             else:
-                f.write(toolchain_ubuntu2004_nompi(gcc_version=gcc_version))
+                f.write(install_deps_ubuntu2004(gcc_version=gcc_version))
+                # Have to use Makefile because Ubuntu:20.04 ships with CMake 3.16.3.
                 # Skip some tests due to bug in LDA_C_PMGB06 functional in libxc <5.2.0.
                 f.write(regtest("ssmp", testopts="--skipdir=QS/regtest-rs-dhft"))
 
-    with OutputFile("Dockerfile.test_i386", args.check) as f:
-        f.write(
-            toolchain_ubuntu_nompi(
-                base_image="i386/debian:12.5", gcc_version=12, with_libxsmm=False
-            )
-        )
-        f.write(regtest_cmake("ubuntu_i386", "ssmp"))
-
     with OutputFile("Dockerfile.test_arm64-psmp", args.check) as f:
-        f.write(
-            toolchain_full(
-                base_image="arm64v8/ubuntu:24.04",
-                with_libtorch="no",
-                with_deepmd="no",
-            )
-        )
+        base_img = "arm64v8/ubuntu:24.04"
+        f.write(install_deps_toolchain(base_img, with_libtorch="no", with_deepmd="no"))
         f.write(regtest("psmp"))
 
     with OutputFile(f"Dockerfile.test_performance", args.check) as f:
-        f.write(toolchain_full() + performance())
+        f.write(install_deps_toolchain())
+        f.write(performance())
 
     for gpu_ver in "P100", "V100", "A100":
         with OutputFile(f"Dockerfile.test_cuda_{gpu_ver}", args.check) as f:
-            f.write(toolchain_cuda(gpu_ver=gpu_ver) + regtest("psmp", "local_cuda"))
+            f.write(install_deps_toolchain_cuda(gpu_ver=gpu_ver))
+            f.write(regtest("psmp", "local_cuda"))
 
         with OutputFile(f"Dockerfile.test_hip_cuda_{gpu_ver}", args.check) as f:
-            f.write(toolchain_hip_cuda(gpu_ver=gpu_ver) + regtest("psmp", "local_hip"))
+            f.write(install_deps_toolchain_hip_cuda(gpu_ver=gpu_ver))
+            f.write(regtest("psmp", "local_hip"))
 
         with OutputFile(f"Dockerfile.test_performance_cuda_{gpu_ver}", args.check) as f:
-            f.write(toolchain_cuda(gpu_ver=gpu_ver) + performance("local_cuda"))
+            f.write(install_deps_toolchain_cuda(gpu_ver=gpu_ver))
+            f.write(performance("local_cuda"))
 
     for gpu_ver in "Mi50", "Mi100":
         with OutputFile(f"Dockerfile.test_hip_rocm_{gpu_ver}", args.check) as f:
             # ROCm containers require --device, which is not available for docker build.
             # https://rocmdocs.amd.com/en/latest/ROCm_Virtualization_Containers/ROCm-Virtualization-&-Containers.html#docker-hub
-            f.write(toolchain_hip_rocm(gpu_ver=gpu_ver))
+            f.write(install_deps_toolchain_hip_rocm(gpu_ver=gpu_ver))
             f.write(regtest_postponed("psmp", "local_hip"))
 
         with OutputFile(f"Dockerfile.build_hip_rocm_{gpu_ver}", args.check) as f:
-            f.write(toolchain_hip_rocm(gpu_ver=gpu_ver) + build("psmp", "local_hip"))
+            f.write(install_deps_toolchain_hip_rocm(gpu_ver=gpu_ver))
+            f.write(build("psmp", "local_hip"))
 
     with OutputFile(f"Dockerfile.test_conventions", args.check) as f:
-        f.write(toolchain_full() + conventions())
+        f.write(install_deps_toolchain())
+        f.write(conventions())
 
     with OutputFile(f"Dockerfile.test_manual", args.check) as f:
-        f.write(toolchain_full() + manual())
+        f.write(install_deps_toolchain())
+        f.write(manual())
 
     with OutputFile(f"Dockerfile.test_precommit", args.check) as f:
         f.write(precommit())
 
-    for name in "aiida", "ase", "gromacs", "i-pi":
+    for name in "ase", "aiida", "i-pi", "phonopy", "gromacs":
         with OutputFile(f"Dockerfile.test_{name}", args.check) as f:
-            f.write(toolchain_ubuntu_nompi() + test_3rd_party(name))
+            f.write(install_deps_toolchain(mpi_mode="no", with_dbcsr=""))
+            f.write(test_3rd_party(name))
 
     for name in "misc", "doxygen":
         with OutputFile(f"Dockerfile.test_{name}", args.check) as f:
@@ -275,7 +279,7 @@ FROM ubuntu:24.04
 WORKDIR /opt/cp2k-precommit
 COPY ./tools/precommit/ /opt/cp2k-precommit/
 RUN ./install_requirements.sh
-ENV PATH="/opt/venv/bin:$PATH"
+ENV PATH="/opt/venv/bin:/opt/cp2k-precommit:$PATH"
 
 # Install sources.
 WORKDIR /opt/cp2k
@@ -399,27 +403,36 @@ COPY ./tools/regtesting ./tools/regtesting
 
 
 # ======================================================================================
-def toolchain_full(
+def install_dbcsr(profile: str, version: str) -> str:
+    return rf"""
+# Install DBCSR
+COPY ./tools/docker/scripts/install_dbcsr.sh ./
+RUN ./install_dbcsr.sh {profile} {version}
+"""
+
+
+# ======================================================================================
+def install_deps_toolchain(
     base_image: str = "ubuntu:24.04",
+    mpi_mode: str = "mpich",
+    with_dbcsr: str = "no",
     with_gcc: str = "system",
-    with_dbcsr: bool = False,
     **kwargs: str,
 ) -> str:
     output = f"\nFROM {base_image}\n\n"
     output += install_toolchain(
-        base_image=base_image, install_all="", with_gcc=with_gcc, **kwargs
+        base_image=base_image,
+        install_all="",
+        mpi_mode=mpi_mode,
+        with_dbcsr=with_dbcsr,
+        with_gcc=with_gcc,
+        **kwargs,
     )
-    if with_dbcsr:
-        output += r"""
-# Install DBCSR
-COPY ./tools/docker/scripts/install_dbcsr.sh ./
-RUN /bin/bash -o pipefail -c "source /opt/cp2k-toolchain/install/setup; ./install_dbcsr.sh"
-"""
     return output
 
 
 # ======================================================================================
-def toolchain_ubuntu_nompi(
+def install_deps_ubuntu(
     base_image: str = "ubuntu:24.04", gcc_version: int = 13, with_libxsmm: bool = True
 ) -> str:
     assert gcc_version > 8
@@ -453,6 +466,7 @@ RUN export DEBIAN_FRONTEND=noninteractive DEBCONF_NONINTERACTIVE_SEEN=true && \
     libopenblas-dev \
     libint2-dev \
     libxc-dev \
+    libhdf5-dev \
     {"libxsmm-dev" if with_libxsmm else ""} \
     libspglib-f08-dev \
    && rm -rf /var/lib/apt/lists/*
@@ -461,16 +475,12 @@ RUN export DEBIAN_FRONTEND=noninteractive DEBCONF_NONINTERACTIVE_SEEN=true && \
 RUN ln -sf /usr/bin/gcc-{gcc_version}      /usr/local/bin/gcc  && \
     ln -sf /usr/bin/g++-{gcc_version}      /usr/local/bin/g++  && \
     ln -sf /usr/bin/gfortran-{gcc_version} /usr/local/bin/gfortran
-
-# Install DBCSR
-COPY ./tools/docker/scripts/install_dbcsr.sh ./
-RUN  ./install_dbcsr.sh
 """
     return output
 
 
 # ======================================================================================
-def toolchain_ubuntu2004_nompi(gcc_version: int = 8) -> str:
+def install_deps_ubuntu2004(gcc_version: int = 8) -> str:
     output = rf"""
 FROM ubuntu:20.04
 
@@ -498,6 +508,7 @@ RUN ln -sf /usr/bin/gcc-{gcc_version}      /usr/local/bin/gcc  && \
         mpi_mode="no",
         with_gcc="system",
         with_cmake="system",
+        with_dbcsr="no",
         with_fftw="system",
         with_openblas="system",
         with_gsl="system",
@@ -513,21 +524,28 @@ RUN ln -sf /usr/bin/gcc-{gcc_version}      /usr/local/bin/gcc  && \
 
 
 # ======================================================================================
-def toolchain_intel() -> str:
+def install_deps_toolchain_intel(
+    base_image: str = "intel/hpckit:2024.2.1-0-devel-ubuntu22.04",
+    with_ifx: str = "no",
+) -> str:
     return rf"""
-FROM intel/hpckit:2024.2.1-0-devel-ubuntu22.04
+FROM {base_image}
 
 """ + install_toolchain(
         base_image="ubuntu",
         install_all="",
+        with_dbcsr="no",
+        with_ifx=with_ifx,
         with_intelmpi="",
         with_mkl="",
+        with_libsmeagol="",
         with_libtorch="no",
+        with_deepmd="no",
     )
 
 
 # ======================================================================================
-def toolchain_nvhpc() -> str:
+def install_deps_toolchain_nvhpc() -> str:
     return rf"""
 FROM ubuntu:22.04
 
@@ -575,7 +593,7 @@ RUN make -j ARCH=Linux-x86-64-nvhpc VERSION=ssmp cp2k
 
 
 # ======================================================================================
-def toolchain_cuda(gpu_ver: str) -> str:
+def install_deps_toolchain_cuda(gpu_ver: str) -> str:
     return rf"""
 FROM nvidia/cuda:11.8.0-devel-ubuntu22.04
 
@@ -595,12 +613,16 @@ RUN apt-get update -qq && apt-get install -qq --no-install-recommends \
    && rm -rf /var/lib/apt/lists/*
 
 """ + install_toolchain(
-        base_image="ubuntu", mpi_mode="mpich", enable_cuda="yes", gpu_ver=gpu_ver
+        base_image="ubuntu",
+        mpi_mode="mpich",
+        enable_cuda="yes",
+        gpu_ver=gpu_ver,
+        with_dbcsr="no",
     )
 
 
 # ======================================================================================
-def toolchain_hip_cuda(gpu_ver: str) -> str:
+def install_deps_toolchain_hip_cuda(gpu_ver: str) -> str:
     return rf"""
 FROM nvidia/cuda:11.8.0-devel-ubuntu22.04
 
@@ -702,12 +724,16 @@ ENV HIP_PLATFORM nvidia
 RUN hipconfig
 
 """ + install_toolchain(
-        base_image="ubuntu", mpi_mode="mpich", enable_hip="yes", gpu_ver=gpu_ver
+        base_image="ubuntu",
+        mpi_mode="mpich",
+        enable_hip="yes",
+        gpu_ver=gpu_ver,
+        with_dbcsr="no",
     )
 
 
 # ======================================================================================
-def toolchain_hip_rocm(gpu_ver: str) -> str:
+def install_deps_toolchain_hip_rocm(gpu_ver: str) -> str:
     return rf"""
 FROM rocm/dev-ubuntu-22.04:5.3.2-complete
 
@@ -727,7 +753,11 @@ ENV HIP_PLATFORM amd
 RUN hipconfig
 
 """ + install_toolchain(
-        base_image="ubuntu", mpi_mode="mpich", enable_hip="yes", gpu_ver=gpu_ver
+        base_image="ubuntu",
+        mpi_mode="mpich",
+        enable_hip="yes",
+        gpu_ver=gpu_ver,
+        with_dbcsr="no",
     )
 
 
@@ -763,7 +793,7 @@ RUN ./install_cp2k_toolchain.sh \
     --dry-run
 
 # Dry-run leaves behind config files for the followup install scripts.
-# This breaks up the lengthy installation into smaller docker build steps.
+# This breaks up the lengthy installation into smaller build steps.
 COPY ./tools/toolchain/scripts/stage0/ ./scripts/stage0/
 RUN  ./scripts/stage0/install_stage0.sh && rm -rf ./build
 
@@ -791,6 +821,9 @@ RUN  ./scripts/stage7/install_stage7.sh && rm -rf ./build
 COPY ./tools/toolchain/scripts/stage8/ ./scripts/stage8/
 RUN  ./scripts/stage8/install_stage8.sh && rm -rf ./build
 
+COPY ./tools/toolchain/scripts/stage9/ ./scripts/stage9/
+RUN  ./scripts/stage9/install_stage9.sh && rm -rf ./build
+
 COPY ./tools/toolchain/scripts/arch_base.tmpl \
      ./tools/toolchain/scripts/generate_arch_files.sh \
      ./scripts/
@@ -799,71 +832,100 @@ RUN ./scripts/generate_arch_files.sh && rm -rf ./build
 
 
 # ======================================================================================
-def spack_env_toolchain() -> str:
+def install_deps_spack(version: str) -> str:
     return rf"""
 FROM ubuntu:24.04
 
-# Install common dependencies as pre-built Ubuntu packages.
+# Install packages required to build the CP2K dependencies with Spack
 RUN apt-get update -qq && apt-get install -qq --no-install-recommends \
-    autoconf \
-    autogen \
-    automake \
-    autotools-dev \
     bzip2 \
     ca-certificates \
+    cmake \
     g++ \
     gcc \
     gfortran \
     git \
-    less \
+    gnupg \
+    hwloc \
+    libhwloc-dev \
+    libssh-dev \
+    libssl-dev \
     libtool \
     libtool-bin \
+    lsb-release \
     make \
-    nano \
     ninja-build \
     patch \
     pkgconf \
     python3 \
+    python3-dev \
+    python3-pip \
+    python3-venv \
     unzip \
     wget \
     xxd \
-    zlib1g-dev \
-    cmake \
-    gnupg \
-    m4 \
     xz-utils \
-    libssl-dev \
-    libssh-dev \
-    hwloc \
-    libhwloc-dev \
-   && rm -rf /var/lib/apt/lists/*
+    zstd && rm -rf /var/lib/apt/lists/*
 
-# Install a recent developer version of Spack.
-WORKDIR /opt/spack
-ARG SPACK_VERSION=501ee68606405ea2ac26d369a5139d75be88dac8
-RUN git init --quiet && \
-    git remote add origin https://github.com/spack/spack.git && \
-    git fetch --quiet --depth 1 origin ${{SPACK_VERSION}} --no-tags && \
-    git checkout --quiet FETCH_HEAD
-ENV PATH="/opt/spack/bin:${{PATH}}"
+# Create and activate a virtual environment for Python packages
+RUN python3 -m venv /opt/venv
+ENV PATH="/opt/venv/bin:${{PATH}}"
+RUN pip3 install --quiet boto3==1.38.11 google-cloud-storage==3.1.0
 
-# Find all external packages and compilers.
+# Retrieve the number of available CPU cores
+ARG NUM_PROCS
+ENV NUM_PROCS=${{NUM_PROCS:-32}}
+
+# Install Spack and Spack packages
+WORKDIR /root/spack
+ARG SPACK_VERSION
+ENV SPACK_VERSION=${{SPACK_VERSION:-1.0.0}}
+ARG SPACK_PACKAGES_VERSION
+ENV SPACK_PACKAGES_VERSION=${{SPACK_PACKAGES_VERSION:-2025.07.0}}
+ARG SPACK_REPO=https://github.com/spack/spack
+ENV SPACK_ROOT=/opt/spack-${{SPACK_VERSION}}
+ARG SPACK_PACKAGES_REPO=https://github.com/spack/spack-packages
+ENV SPACK_PACKAGES_ROOT=/opt/spack-packages-${{SPACK_PACKAGES_VERSION}}
+RUN mkdir -p ${{SPACK_ROOT}} \
+    && wget -q ${{SPACK_REPO}}/archive/v${{SPACK_VERSION}}.tar.gz \
+    && tar -xzf v${{SPACK_VERSION}}.tar.gz -C /opt && rm -f v${{SPACK_VERSION}}.tar.gz \
+    && mkdir -p ${{SPACK_PACKAGES_ROOT}} \
+    && wget -q ${{SPACK_PACKAGES_REPO}}/archive/v${{SPACK_PACKAGES_VERSION}}.tar.gz \
+    && tar -xzf v${{SPACK_PACKAGES_VERSION}}.tar.gz -C /opt && rm -f v${{SPACK_PACKAGES_VERSION}}.tar.gz
+
+ENV PATH="${{SPACK_ROOT}}/bin:${{PATH}}"
+
+# Add Spack packages builtin repository
+RUN spack repo add --scope site ${{SPACK_PACKAGES_ROOT}}/repos/spack_repo/builtin
+
+# Find all compilers
 RUN spack compiler find
+
+# Find all external packages
 RUN spack external find --all --not-buildable
 
-# Enable Spack build cache
-ARG SPACK_BUILD_CACHE=develop-2024-12-15
-RUN spack mirror add ${{SPACK_BUILD_CACHE}} https://binaries.spack.io/${{SPACK_BUILD_CACHE}} && \
-    spack mirror add develop https://binaries.spack.io/develop && \
-    spack buildcache keys --install --trust --force && \
-    spack mirror rm develop
+# Add local Spack cache
+ARG SPACK_CACHE="s3://spack-cache --s3-endpoint-url=http://localhost:9000"
+COPY ./tools/docker/scripts/setup_spack_cache.sh ./
+RUN ./setup_spack_cache.sh
 
-# Install CP2K's dependencies via Spack.
-WORKDIR /
-COPY ./tools/spack/cp2k-dependencies.yaml .
-RUN spack env create myenv ./cp2k-dependencies.yaml
+# Copy Spack configuration and build recipes
+ARG CP2K_VERSION
+ENV CP2K_VERSION=${{CP2K_VERSION:-{version}}}
+ARG CP2K_BUILD_TYPE
+ENV CP2K_BUILD_TYPE=${{CP2K_BUILD_TYPE:-all}}
+COPY ./tools/spack/cp2k_deps_${{CP2K_BUILD_TYPE}}_${{CP2K_VERSION}}.yaml ./
+COPY ./tools/spack/cp2k_dev_repo ${{SPACK_PACKAGES_ROOT}}/repos/spack_repo/cp2k_dev_repo/
+RUN spack repo add --scope site ${{SPACK_PACKAGES_ROOT}}/repos/spack_repo/cp2k_dev_repo/
+RUN spack env create myenv cp2k_deps_${{CP2K_BUILD_TYPE}}_${{CP2K_VERSION}}.yaml && \
+    spack -e myenv repo list
+
+# Install CP2K dependencies via Spack
 RUN spack -e myenv concretize -f
-RUN spack -e myenv env depfile -o spack-makefile && make -j32 --file=spack-makefile SPACK_COLOR=never --output-sync=recurse
+ENV SPACK_ENV_VIEW="${{SPACK_ROOT}}/var/spack/environments/myenv/spack-env/view"
+RUN spack -e myenv env depfile -o spack_makefile && \
+    make -j${{NUM_PROCS}} --file=spack_makefile SPACK_COLOR=never --output-sync=recurse && \
+    cp -ar ${{SPACK_ENV_VIEW}}/bin ${{SPACK_ENV_VIEW}}/include ${{SPACK_ENV_VIEW}}/lib /opt/spack
 """
 
 
@@ -875,7 +937,9 @@ class OutputFile:
         self.content = io.StringIO()
         self.content.write(f"#\n")
         self.content.write(f"# This file was created by generate_dockerfiles.py.\n")
-        self.content.write(f"# Usage: docker build -f ./{filename} ../../\n")
+        self.content.write(
+            f"# Usage: podman build --shm-size=1g -f ./{filename} ../../\n"
+        )
         self.content.write(f"#\n")
 
     def __enter__(self) -> io.StringIO:
