@@ -3,7 +3,7 @@
 # author: Ole Schuett
 
 # Compile CP2K.
-./build_cp2k_cmake.sh "ubuntu" "ssmp" || exit 0
+./build_cp2k_cmake.sh "toolchain_all" "ssmp" || exit 0
 
 # Fake installation of data files.
 mkdir -p ./share/cp2k
@@ -12,7 +12,8 @@ ln -s ../../data ./share/cp2k/data
 cat > /usr/bin/cp2k_shell << EndOfMessage
 #!/bin/bash -e
 export OMP_NUM_THREADS=1
-/opt/cp2k/exe/local/cp2k.ssmp --shell "\$@"
+source /opt/cp2k-toolchain/install/setup
+/opt/cp2k/build/bin/cp2k.ssmp --shell "\$@"
 EndOfMessage
 chmod +x /usr/bin/cp2k_shell
 
@@ -21,7 +22,8 @@ chmod +x /usr/bin/cp2k_shell
 cat > /usr/bin/cp2k << EndOfMessage
 #!/bin/bash -e
 export OMP_NUM_THREADS=1
-/opt/cp2k/exe/local/cp2k.ssmp "\$@"
+source /opt/cp2k-toolchain/install/setup
+/opt/cp2k/build/bin/cp2k.ssmp "\$@"
 EndOfMessage
 chmod +x /usr/bin/cp2k
 
