@@ -18,24 +18,30 @@ T = TypeVar("T")
 CP2K_DIR = pathlib.Path(__file__).resolve().parents[2]
 
 FLAG_EXCEPTIONS = (
-    r"\$\{..*\}\$",
-    r"__..*__",
-    r"_M_..*",
+    r"\$\{.+\}\$",
+    r"__.+__",
+    r"_M_.+",
     r"__ARM_ARCH",
-    r"__ARM_FEATURE_..*",
+    r"__ARM_FEATURE_.+",
     r"CUDA_VERSION",
     r"DBM_LIBXSMM_PREFETCH",
-    r"DBM_VALIDATE_AGAINST_DBCSR",
+    r"DBM_VALIDATE_AGAINST_.+",
+    r"DBM_ALLOC_.+",
+    r"DBM_MEMPOOL_.+",
     r"OPENMP_TRACE_SYMBOL",
-    r"OPENCL_DBM_..*",
+    r"OPENCL_.+",
+    r"ACC_OPENCL_.+",
     r"FD_DEBUG",
     r"GRID_DO_COLLOCATE",
     r"INTEL_MKL_VERSION",
     r"LIBINT2_MAX_AM_eri",
+    r"LIBGRPP",
+    r"M_",
     r"LIBINT_CONTRACTED_INTS",
     r"XC_MAJOR_VERSION",
     r"XC_MINOR_VERSION",
     r"NDEBUG",
+    r"M_PI",
     r"OMP_DEFAULT_NONE_WITH_OOP",
     r"FTN_NO_DEFAULT_INIT",
     r"_OPENMP",
@@ -67,6 +73,8 @@ FLAG_EXCEPTIONS = (
     r"LIBXSMM_VERSION2",
     r"LIBXSMM_VERSION3",
     r"LIBXSMM_VERSION4",
+    r"LIBGRPP_.+",
+    r"TEST_LIBGRPP_.+",
     r"__LIBXSMM2",
     r"CPVERSION",
     r"_WIN32",
@@ -117,6 +125,8 @@ C_EXTENSIONS = (".c", ".cu", ".cpp", ".cc", ".h", ".hpp")
 
 BSD_PATHS = ("src/offload/", "src/grid/", "src/dbm/", "src/base/openmp_trace.c")
 
+MIT_PATHS = ("src/grpp/",)
+
 
 @lru_cache(maxsize=None)
 def get_install_txt() -> str:
@@ -147,7 +157,7 @@ def check_file(path: pathlib.Path) -> List[str]:
     - undocumented preprocessor flags
     - stray unicode characters
     """
-    warnings = []
+    warnings: List[str] = []
 
     fn_ext = path.suffix
     abspath = path.resolve()
@@ -191,7 +201,14 @@ def check_file(path: pathlib.Path) -> List[str]:
     # check banner
     year = datetime.now(timezone.utc).year
     bsd_licensed = any(str(path).startswith(p) for p in BSD_PATHS)
-    spdx = "BSD-3-Clause    " if bsd_licensed else "GPL-2.0-or-later"
+    mit_licensed = any(str(path).startswith(p) for p in MIT_PATHS)
+    if bsd_licensed:
+        spdx = "BSD-3-Clause    "
+    elif mit_licensed:
+        spdx = "MIT             "
+    else:
+        spdx = "GPL-2.0-or-later"
+
     if fn_ext == ".F" and not content.startswith(BANNER_F.format(year, spdx)):
         warnings += [f"{path}: Copyright banner malformed"]
     if fn_ext == ".fypp" and not content.startswith(BANNER_SHELL.format(year, spdx)):
@@ -210,7 +227,6 @@ def check_file(path: pathlib.Path) -> List[str]:
     PY_SHEBANG = "#!/usr/bin/env python3"
     if fn_ext == ".py" and is_executable and not content.startswith(f"{PY_SHEBANG}\n"):
         warnings += [f"{path}: Wrong shebang, please use '{PY_SHEBANG}'"]
-
     # find all flags
     flags = set()
     line_continuation = False
