@@ -124,11 +124,12 @@ def main() -> None:
                 continue
             file_list += [os.path.join(root, fn) for fn in files]
 
-    # Filter symlinks, backup copies, logs, and hidden files.
+    # Filter symlinks, backup copies, logs, hidden, and very large files.
     file_list = [fn for fn in file_list if not os.path.islink(fn)]
     file_list = [fn for fn in file_list if not fn[-1] in ("~", "#")]
     file_list = [fn for fn in file_list if not fn.endswith(".log")]
     file_list = [fn for fn in file_list if not os.path.basename(fn).startswith(".")]
+    file_list = [fn for fn in file_list if os.path.getsize(fn) < 2**20]  # 1MiB
 
     # Sort files by size as larger ones will take longer to process.
     file_list.sort(reverse=True, key=lambda fn: os.path.getsize(fn))
@@ -207,12 +208,18 @@ def process_file(fn: str, allow_modifications: bool) -> None:
         if fn.endswith("/torch_c_api.cpp"):
             # Begrudgingly tolerated because PyTorch has no C API.
             run_remote_tool("clangformat", fn)
+        elif fn.endswith("/ace_c_api.cpp"):
+            # same as PyTorch
+            run_remote_tool("clangformat", fn)
         else:
             raise Exception(f"C++ is not supported.")
 
     if re.match(r"(.*/PACKAGE)|(.*\.py)$", fn):
         ast.parse(orig_content, filename=fn)
-        run_remote_tool("black", fn)
+        if "tools/spack/cp2k_dev_repo/packages" in fn:
+            run_remote_tool("spackformat", fn)
+        else:
+            run_remote_tool("black", fn)
 
     if re.match(r".*\.sh$", fn):
         run_remote_tool("shfmt", fn)
